==> ideaIC-171.3780.95-sources.jar: com/intellij/openapi/application/ReadAction.java <==
/*
 * Copyright 2000-2016 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.intellij.openapi.application;

import com.intellij.openapi.util.Computable;
import com.intellij.openapi.util.ThrowableComputable;
import com.intellij.util.ThrowableRunnable;
import org.jetbrains.annotations.NotNull;

public abstract class ReadAction<T> extends BaseActionRunnable<T> {
  @NotNull
  @Override
  public RunResult<T> execute() {
    final RunResult<T> result = new RunResult<>(this);
    return ApplicationManager.getApplication().runReadAction((Computable<RunResult<T>>)() -> result.run());
  }

  public static AccessToken start() {
    return ApplicationManager.getApplication().acquireReadActionLock();
  }

  public static <E extends Throwable> void run(@NotNull ThrowableRunnable<E> action) throws E {
    AccessToken token = start();
    try {
      action.run();
    }
    finally {
      token.finish();
    }
  }

  public static <T, E extends Throwable> T compute(@NotNull ThrowableComputable<T, E> action) throws E {
    AccessToken token = start();
    try {
      return action.compute();
    }
    finally {
      token.finish();
    }
  }
}

==> ideaIC-173.4301.1-sources.jar: com/intellij/openapi/application/ReadAction.java <==
/*
 * Copyright 2000-2016 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.intellij.openapi.application;

import com.intellij.openapi.util.ThrowableComputable;
import com.intellij.util.ThrowableRunnable;
import org.jetbrains.annotations.NotNull;

public abstract class ReadAction<T> extends BaseActionRunnable<T> {
  @NotNull
  @Override
  public RunResult<T> execute() {
    final RunResult<T> result = new RunResult<>(this);
    return ReadAction.compute(() -> result.run());
  }

  public static AccessToken start() {
    return ApplicationManager.getApplication().acquireReadActionLock();
  }

  public static <E extends Throwable> void run(@NotNull ThrowableRunnable<E> action) throws E {
    AccessToken token = start();
    try {
      action.run();
    }
    finally {
      token.finish();
    }
  }

  public static <T, E extends Throwable> T compute(@NotNull ThrowableComputable<T, E> action) throws E {
    AccessToken token = start();
    try {
      return action.compute();
    }
    finally {
      token.finish();
    }
  }
}
